@@ -84,7 +84,7 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 			aerc.NewTab(composer, tabName)
 		}
 
-		aerc.RemoveTab(composer)
+		aerc.RemoveTab(composer, false)
 		var buf bytes.Buffer
 		ctr := datacounter.NewWriterCounter(&buf)
 		err = composer.WriteMessage(header, ctr)
@@ -95,7 +95,7 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 		nbytes := int(ctr.Count())
 		worker.PostAction(&types.AppendMessage{
 			Destination: config.Postpone,
-			Flags:       []models.Flag{models.SeenFlag},
+			Flags:       models.SeenFlag,
 			Date:        time.Now(),
 			Reader:      &buf,
 			Length:      int(nbytes),
