@@ -31,21 +31,14 @@ func (MoveTab) Execute(aerc *widgets.Aerc, args []string) error {
 
 	n, err := strconv.Atoi(joinedArgs)
 	if err != nil {
-		return fmt.Errorf("failed to parse index argument: %v", err)
+		return fmt.Errorf("failed to parse index argument: %w", err)
 	}
 
-	i := aerc.SelectedTabIndex()
-	l := aerc.NumTabs()
-
-	if strings.HasPrefix(joinedArgs, "+") {
-		i = (i + n) % l
-	} else if strings.HasPrefix(joinedArgs, "-") {
-		i = (((i + n) % l) + l) % l
-	} else {
-		i = n
+	var relative bool
+	if strings.HasPrefix(joinedArgs, "+") || strings.HasPrefix(joinedArgs, "-") {
+		relative = true
 	}
-
-	aerc.MoveTab(i)
+	aerc.MoveTab(n, relative)
 
 	return nil
 }
