@@ -9,6 +9,7 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/emersion/go-message/mail"
 	"github.com/gdamore/tcell/v2"
 	"github.com/go-ini/ini"
 	"github.com/mitchellh/go-homedir"
@@ -37,6 +38,7 @@ const (
 	STYLE_MSGLIST_DELETED
 	STYLE_MSGLIST_MARKED
 	STYLE_MSGLIST_RESULT
+	STYLE_MSGLIST_ANSWERED
 
 	STYLE_DIRLIST_DEFAULT
 	STYLE_DIRLIST_UNREAD
@@ -70,13 +72,14 @@ var StyleNames = map[string]StyleObject{
 	"statusline_warning": STYLE_STATUSLINE_WARNING,
 	"statusline_success": STYLE_STATUSLINE_SUCCESS,
 
-	"msglist_default": STYLE_MSGLIST_DEFAULT,
-	"msglist_unread":  STYLE_MSGLIST_UNREAD,
-	"msglist_read":    STYLE_MSGLIST_READ,
-	"msglist_flagged": STYLE_MSGLIST_FLAGGED,
-	"msglist_deleted": STYLE_MSGLIST_DELETED,
-	"msglist_marked":  STYLE_MSGLIST_MARKED,
-	"msglist_result":  STYLE_MSGLIST_RESULT,
+	"msglist_default":  STYLE_MSGLIST_DEFAULT,
+	"msglist_unread":   STYLE_MSGLIST_UNREAD,
+	"msglist_read":     STYLE_MSGLIST_READ,
+	"msglist_flagged":  STYLE_MSGLIST_FLAGGED,
+	"msglist_deleted":  STYLE_MSGLIST_DELETED,
+	"msglist_marked":   STYLE_MSGLIST_MARKED,
+	"msglist_result":   STYLE_MSGLIST_RESULT,
+	"msglist_answered": STYLE_MSGLIST_ANSWERED,
 
 	"dirlist_default": STYLE_DIRLIST_DEFAULT,
 	"dirlist_unread":  STYLE_DIRLIST_UNREAD,
@@ -104,6 +107,10 @@ type Style struct {
 	Underline bool
 	Reverse   bool
 	Italic    bool
+	Dim       bool
+	header    string         // only for msglist
+	pattern   string         // only for msglist
+	re        *regexp.Regexp // only for msglist
 }
 
 func (s Style) Get() tcell.Style {
@@ -114,7 +121,8 @@ func (s Style) Get() tcell.Style {
 		Blink(s.Blink).
 		Underline(s.Underline).
 		Reverse(s.Reverse).
-		Italic(s.Italic)
+		Italic(s.Italic).
+		Dim(s.Dim)
 }
 
 func (s *Style) Normal() {
@@ -123,6 +131,7 @@ func (s *Style) Normal() {
 	s.Underline = false
 	s.Reverse = false
 	s.Italic = false
+	s.Dim = false
 }
 
 func (s *Style) Default() *Style {
@@ -198,6 +207,12 @@ func (s *Style) Set(attr, val string) error {
 		} else {
 			s.Italic = state
 		}
+	case "dim":
+		if state, err := boolSwitch(val, s.Dim); err != nil {
+			return err
+		} else {
+			s.Dim = state
+		}
 	case "default":
 		s.Default()
 	case "normal":
@@ -212,10 +227,10 @@ func (s *Style) Set(attr, val string) error {
 func (s Style) composeWith(styles []*Style) Style {
 	newStyle := s
 	for _, st := range styles {
-		if st.Fg != s.Fg {
+		if st.Fg != s.Fg && st.Fg != tcell.ColorDefault {
 			newStyle.Fg = st.Fg
 		}
-		if st.Bg != s.Bg {
+		if st.Bg != s.Bg && st.Bg != tcell.ColorDefault {
 			newStyle.Bg = st.Bg
 		}
 		if st.Bold != s.Bold {
@@ -233,60 +248,99 @@ func (s Style) composeWith(styles []*Style) Style {
 		if st.Italic != s.Italic {
 			newStyle.Italic = st.Italic
 		}
+		if st.Dim != s.Dim {
+			newStyle.Dim = st.Dim
+		}
 	}
 	return newStyle
 }
 
+type StyleConf struct {
+	base    Style
+	dynamic []Style
+}
+
 type StyleSet struct {
-	objects  map[StyleObject]*Style
-	selected map[StyleObject]*Style
+	objects  map[StyleObject]*StyleConf
+	selected map[StyleObject]*StyleConf
+	user     map[string]*Style
+	path     string
 }
 
 func NewStyleSet() StyleSet {
 	ss := StyleSet{
-		objects:  make(map[StyleObject]*Style),
-		selected: make(map[StyleObject]*Style),
+		objects:  make(map[StyleObject]*StyleConf),
+		selected: make(map[StyleObject]*StyleConf),
+		user:     make(map[string]*Style),
 	}
 	for _, so := range StyleNames {
-		ss.objects[so] = new(Style)
-		ss.selected[so] = new(Style)
+		ss.objects[so] = new(StyleConf)
+		ss.selected[so] = new(StyleConf)
 	}
-
 	return ss
 }
 
 func (ss StyleSet) reset() {
 	for _, so := range StyleNames {
-		ss.objects[so].Reset()
-		ss.selected[so].Reset()
+		ss.objects[so].base.Reset()
+		for _, d := range ss.objects[so].dynamic {
+			d.Reset()
+		}
+		ss.selected[so].base.Reset()
+		for _, d := range ss.selected[so].dynamic {
+			d.Reset()
+		}
 	}
 }
 
-func (ss StyleSet) Get(so StyleObject) tcell.Style {
-	return ss.objects[so].Get()
+func (c *StyleConf) getStyle(h *mail.Header) *Style {
+	if h == nil {
+		return &c.base
+	}
+	for _, s := range c.dynamic {
+		val, _ := h.Text(s.header)
+		if s.re.MatchString(val) {
+			s = c.base.composeWith([]*Style{&s})
+			return &s
+		}
+	}
+	return &c.base
 }
 
-func (ss StyleSet) Selected(so StyleObject) tcell.Style {
-	return ss.selected[so].Get()
+func (ss StyleSet) Get(so StyleObject, h *mail.Header) tcell.Style {
+	return ss.objects[so].getStyle(h).Get()
 }
 
-func (ss StyleSet) Compose(so StyleObject, sos []StyleObject) tcell.Style {
-	base := *ss.objects[so]
+func (ss StyleSet) Selected(so StyleObject, h *mail.Header) tcell.Style {
+	return ss.selected[so].getStyle(h).Get()
+}
+
+func (ss StyleSet) UserStyle(name string) tcell.Style {
+	if style, found := ss.user[name]; found {
+		return style.Get()
+	}
+	return tcell.StyleDefault
+}
+
+func (ss StyleSet) Compose(
+	so StyleObject, sos []StyleObject, h *mail.Header,
+) tcell.Style {
+	base := *ss.objects[so].getStyle(h)
 	styles := make([]*Style, len(sos))
 	for i, so := range sos {
-		styles[i] = ss.objects[so]
+		styles[i] = ss.objects[so].getStyle(h)
 	}
 
 	return base.composeWith(styles).Get()
 }
 
-func (ss StyleSet) ComposeSelected(so StyleObject,
-	sos []StyleObject,
+func (ss StyleSet) ComposeSelected(
+	so StyleObject, sos []StyleObject, h *mail.Header,
 ) tcell.Style {
-	base := *ss.selected[so]
+	base := *ss.selected[so].getStyle(h)
 	styles := make([]*Style, len(sos))
 	for i, so := range sos {
-		styles[i] = ss.selected[so]
+		styles[i] = ss.selected[so].getStyle(h)
 	}
 
 	return base.composeWith(styles).Get()
@@ -318,110 +372,118 @@ func (ss *StyleSet) ParseStyleSet(file *ini.File) error {
 		return err
 	}
 
-	selectedKeys := []string{}
+	// parse non-selected items first
+	for _, key := range defaultSection.Keys() {
+		err = ss.parseKey(key, false)
+		if err != nil {
+			return err
+		}
+	}
+	// override with selected items afterwards
+	for _, key := range defaultSection.Keys() {
+		err = ss.parseKey(key, true)
+		if err != nil {
+			return err
+		}
+	}
 
-	for _, key := range defaultSection.KeyStrings() {
+	user, err := file.GetSection("user")
+	if err != nil {
+		// This errors if the section doesn't exist, which is ok
+		return nil
+	}
+	for _, key := range user.KeyStrings() {
 		tokens := strings.Split(key, ".")
 		var styleName, attr string
 		switch len(tokens) {
 		case 2:
 			styleName, attr = tokens[0], tokens[1]
-		case 3:
-			if tokens[1] != "selected" {
-				return errors.New("Unknown modifier: " + tokens[1])
-			}
-			selectedKeys = append(selectedKeys, key)
-			continue
 		default:
 			return errors.New("Style parsing error: " + key)
 		}
-		val := defaultSection.KeysHash()[key]
-
-		if strings.ContainsAny(styleName, "*?") {
-			regex := fnmatchToRegex(styleName)
-			for sn, so := range StyleNames {
-				matched, err := regexp.MatchString(regex, sn)
-				if err != nil {
-					return err
-				}
-
-				if !matched {
-					continue
-				}
-
-				if err := ss.objects[so].Set(attr, val); err != nil {
-					return err
-				}
-				if err := ss.selected[so].Set(attr, val); err != nil {
-					return err
-				}
-			}
-		} else {
-			so, ok := StyleNames[styleName]
-			if !ok {
-				return errors.New("Unknown style object: " + styleName)
-			}
-			if err := ss.objects[so].Set(attr, val); err != nil {
-				return err
-			}
-			if err := ss.selected[so].Set(attr, val); err != nil {
-				return err
-			}
+		val := user.KeysHash()[key]
+		s, ok := ss.user[styleName]
+		if !ok {
+			// Haven't seen this name before, add it to the map
+			s = &Style{}
+			ss.user[styleName] = s
+		}
+		if err := s.Set(attr, val); err != nil {
+			return err
 		}
 	}
 
-	for _, key := range selectedKeys {
-		tokens := strings.Split(key, ".")
-		styleName, modifier, attr := tokens[0], tokens[1], tokens[2]
-		if modifier != "selected" {
-			return errors.New("Unknown modifier: " + modifier)
-		}
+	return nil
+}
 
-		val := defaultSection.KeysHash()[key]
+var styleObjRe = regexp.MustCompile(`^([\w\*\?]+)(?:\.([\w-]+),(.+?))?(\.selected)?\.(\w+)$`)
 
-		if strings.ContainsAny(styleName, "*?") {
-			regex := fnmatchToRegex(styleName)
-			for sn, so := range StyleNames {
-				matched, err := regexp.MatchString(regex, sn)
-				if err != nil {
-					return err
-				}
-
-				if !matched {
-					continue
-				}
-
-				if err := ss.selected[so].Set(attr, val); err != nil {
-					return err
-				}
-			}
-		} else {
-			so, ok := StyleNames[styleName]
-			if !ok {
-				return errors.New("Unknown style object: " + styleName)
-			}
-			if err := ss.selected[so].Set(attr, val); err != nil {
-				return err
-			}
-		}
+func (ss *StyleSet) parseKey(key *ini.Key, selected bool) error {
+	groups := styleObjRe.FindStringSubmatch(key.Name())
+	if groups == nil {
+		return errors.New("invalid style syntax: " + key.Name())
 	}
+	if (groups[4] == ".selected") != selected {
+		return nil
+	}
+	obj, attr := groups[1], groups[5]
+	header, pattern := groups[2], groups[3]
 
-	for _, key := range defaultSection.KeyStrings() {
-		tokens := strings.Split(key, ".")
-		styleName, attr := tokens[0], tokens[1]
-		val := defaultSection.KeysHash()[key]
-
-		if styleName != "selected" {
+	objRe, err := fnmatchToRegex(obj)
+	if err != nil {
+		return err
+	}
+	num := 0
+	for sn, so := range StyleNames {
+		if !objRe.MatchString(sn) {
 			continue
 		}
-
-		for _, so := range StyleNames {
-			if err := ss.selected[so].Set(attr, val); err != nil {
+		if !selected {
+			err = ss.objects[so].update(header, pattern, attr, key.Value())
+			if err != nil {
 				return err
 			}
 		}
+		err = ss.selected[so].update(header, pattern, attr, key.Value())
+		if err != nil {
+			return err
+		}
+		num++
 	}
+	if num == 0 {
+		return errors.New("unknown style object: " + obj)
+	}
+	return nil
+}
 
+func (c *StyleConf) update(header, pattern, attr, val string) error {
+	if header == "" || pattern == "" {
+		return (&c.base).Set(attr, val)
+	}
+	for i := range c.dynamic {
+		s := &c.dynamic[i]
+		if s.header == header && s.pattern == pattern {
+			return s.Set(attr, val)
+		}
+	}
+	if strings.HasPrefix(pattern, "~") {
+		pattern = pattern[1:]
+	} else {
+		pattern = "^" + regexp.QuoteMeta(pattern) + "$"
+	}
+	re, err := regexp.Compile(pattern)
+	if err != nil {
+		return err
+	}
+	var s Style
+	err = (&s).Set(attr, val)
+	if err != nil {
+		return err
+	}
+	s.header = header
+	s.pattern = pattern
+	s.re = re
+	c.dynamic = append(c.dynamic, s)
 	return nil
 }
 
@@ -439,23 +501,13 @@ func (ss *StyleSet) LoadStyleSet(stylesetName string, stylesetDirs []string) err
 		return err
 	}
 
+	ss.path = filepath
+
 	return ss.ParseStyleSet(file)
 }
 
-func fnmatchToRegex(pattern string) string {
-	n := len(pattern)
-	var regex strings.Builder
-
-	for i := 0; i < n; i++ {
-		switch pattern[i] {
-		case '*':
-			regex.WriteString(".*")
-		case '?':
-			regex.WriteByte('.')
-		default:
-			regex.WriteByte(pattern[i])
-		}
-	}
-
-	return regex.String()
+func fnmatchToRegex(pattern string) (*regexp.Regexp, error) {
+	p := regexp.QuoteMeta(pattern)
+	p = strings.ReplaceAll(p, `\*`, `.*`)
+	return regexp.Compile(strings.ReplaceAll(p, `\?`, `.`))
 }
