@@ -1,6 +1,7 @@
 package widgets
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
 	"sync"
@@ -10,9 +11,11 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/hooks"
 	"git.sr.ht/~rjarry/aerc/lib/marker"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
-	"git.sr.ht/~rjarry/aerc/lib/statusline"
+	"git.sr.ht/~rjarry/aerc/lib/state"
+	"git.sr.ht/~rjarry/aerc/lib/templates"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
@@ -30,9 +33,10 @@ type AccountView struct {
 	labels  []string
 	grid    *ui.Grid
 	host    TabHost
+	tab     *ui.Tab
 	msglist *MessageList
 	worker  *types.Worker
-	state   *statusline.State
+	state   state.AccountState
 	newConn bool // True if this is a first run after a new connection/reconnection
 	uiConf  *config.UIConfig
 
@@ -63,7 +67,6 @@ func NewAccountView(
 		acct:   acct,
 		aerc:   aerc,
 		host:   host,
-		state:  statusline.NewState(acct.Name, len(config.Accounts) > 1),
 		uiConf: acctUiConf,
 	}
 
@@ -104,7 +107,7 @@ func NewAccountView(
 
 	worker.PostAction(&types.Configure{Config: acct}, nil)
 	worker.PostAction(&types.Connect{}, nil)
-	view.SetStatus(statusline.ConnectionActivity("Connecting..."))
+	view.SetStatus(state.ConnectionActivity("Connecting..."))
 	if acct.CheckMail.Minutes() > 0 {
 		view.CheckMailTimer(acct.CheckMail)
 	}
@@ -112,16 +115,16 @@ func NewAccountView(
 	return view, nil
 }
 
-func (acct *AccountView) SetStatus(setters ...statusline.SetStateFunc) {
+func (acct *AccountView) SetStatus(setters ...state.SetStateFunc) {
 	for _, fn := range setters {
-		fn(acct.state, acct.SelectedDirectory())
+		fn(&acct.state, acct.SelectedDirectory())
 	}
 	acct.UpdateStatus()
 }
 
 func (acct *AccountView) UpdateStatus() {
 	if acct.isSelected() {
-		acct.host.SetStatus(acct.state.StatusLine(acct.SelectedDirectory()))
+		acct.host.UpdateStatus()
 	}
 }
 
@@ -154,9 +157,6 @@ func (acct *AccountView) Invalidate() {
 }
 
 func (acct *AccountView) Draw(ctx *ui.Context) {
-	if acct.state.SetWidth(ctx.Width()) {
-		acct.UpdateStatus()
-	}
 	acct.grid.Draw(ctx)
 }
 
@@ -196,6 +196,9 @@ func (acct *AccountView) SelectedDirectory() string {
 }
 
 func (acct *AccountView) SelectedMessage() (*models.MessageInfo, error) {
+	if acct.msglist == nil || acct.msglist.Store() == nil {
+		return nil, errors.New("init in progress")
+	}
 	if len(acct.msglist.Store().Uids()) == 0 {
 		return nil, errors.New("no message selected")
 	}
@@ -227,32 +230,16 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 	case *types.Done:
 		switch msg.InResponseTo().(type) {
 		case *types.Connect, *types.Reconnect:
-			acct.SetStatus(statusline.ConnectionActivity("Listing mailboxes..."))
+			acct.SetStatus(state.ConnectionActivity("Listing mailboxes..."))
+			log.Infof("[%s] connected.", acct.acct.Name)
+			acct.SetStatus(state.SetConnected(true))
 			log.Tracef("Listing mailboxes...")
-			acct.dirlist.UpdateList(func(dirs []string) {
-				var dir string
-				for _, _dir := range dirs {
-					if _dir == acct.acct.Default {
-						dir = _dir
-						break
-					}
-				}
-				if dir == "" && len(dirs) > 0 {
-					dir = dirs[0]
-				}
-				if dir != "" {
-					acct.dirlist.Select(dir)
-				}
-				acct.msglist.SetInitDone()
-				log.Infof("[%s] connected.", acct.acct.Name)
-				acct.SetStatus(statusline.SetConnected(true))
-				acct.newConn = true
-			})
+			acct.worker.PostAction(&types.ListDirectories{}, nil)
 		case *types.Disconnect:
 			acct.dirlist.ClearList()
 			acct.msglist.SetStore(nil)
 			log.Infof("[%s] disconnected.", acct.acct.Name)
-			acct.SetStatus(statusline.SetConnected(false))
+			acct.SetStatus(state.SetConnected(false))
 		case *types.OpenDirectory:
 			if store, ok := acct.dirlist.SelectedMsgStore(); ok {
 				// If we've opened this dir before, we can re-render it from
@@ -260,50 +247,77 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 				// snappier. If not, we'll unset the store and show the spinner
 				// while we download the UID list.
 				acct.msglist.SetStore(store)
+				acct.Store().Update(msg.InResponseTo())
 			} else {
 				acct.msglist.SetStore(nil)
 			}
 		case *types.CreateDirectory:
-			acct.dirlist.UpdateList(nil)
+			acct.dirlist.Update(msg)
 		case *types.RemoveDirectory:
-			acct.dirlist.UpdateList(nil)
+			acct.dirlist.Update(msg)
 		case *types.FetchMessageHeaders:
 			if acct.newConn {
 				acct.checkMailOnStartup()
 			}
+		case *types.ListDirectories:
+			acct.dirlist.Update(msg)
+			if acct.dirlist.Selected() != "" {
+				return
+			}
+			// Nothing selected, select based on config
+			dirs := acct.dirlist.List()
+			var dir string
+			for _, _dir := range dirs {
+				if _dir == acct.acct.Default {
+					dir = _dir
+					break
+				}
+			}
+			if dir == "" && len(dirs) > 0 {
+				dir = dirs[0]
+			}
+			if dir != "" {
+				acct.dirlist.Select(dir)
+			}
+			acct.msglist.SetInitDone()
+			acct.newConn = true
 		}
+	case *types.Directory:
+		name := msg.Dir.Name
+		store := lib.NewMessageStore(acct.worker,
+			acct.GetSortCriteria(),
+			acct.dirlist.UiConfig(name).ThreadingEnabled,
+			acct.dirlist.UiConfig(name).ForceClientThreads,
+			acct.dirlist.UiConfig(name).ClientThreadsDelay,
+			acct.dirlist.UiConfig(name).ReverseOrder,
+			acct.dirlist.UiConfig(name).ReverseThreadOrder,
+			acct.dirlist.UiConfig(name).SortThreadSiblings,
+			func(msg *models.MessageInfo) {
+				err := hooks.RunHook(&hooks.MailReceived{
+					MsgInfo: msg,
+				})
+				if err != nil {
+					msg := fmt.Sprintf("mail-received hook: %s", err)
+					acct.aerc.PushError(msg)
+				}
+			}, func() {
+				if acct.dirlist.UiConfig(name).NewMessageBell {
+					acct.host.Beep()
+				}
+			},
+			acct.updateSplitView,
+		)
+		store.SetMarker(marker.New(store))
+		acct.dirlist.SetMsgStore(msg.Dir, store)
 	case *types.DirectoryInfo:
-		if store, ok := acct.dirlist.MsgStore(msg.Info.Name); ok {
-			store.Update(msg)
-		} else {
-			name := msg.Info.Name
-			store = lib.NewMessageStore(acct.worker, msg.Info,
-				acct.GetSortCriteria(),
-				acct.dirlist.UiConfig(name).ThreadingEnabled,
-				acct.dirlist.UiConfig(name).ForceClientThreads,
-				acct.dirlist.UiConfig(name).ClientThreadsDelay,
-				acct.dirlist.UiConfig(name).ReverseOrder,
-				acct.dirlist.UiConfig(name).ReverseThreadOrder,
-				acct.dirlist.UiConfig(name).SortThreadSiblings,
-				func(msg *models.MessageInfo) {
-					config.Triggers.ExecNewEmail(acct.acct, msg)
-				}, func() {
-					if acct.dirlist.UiConfig(name).NewMessageBell {
-						acct.host.Beep()
-					}
-				},
-				acct.updateSplitView,
-			)
-			store.SetMarker(marker.New(store))
-			acct.dirlist.SetMsgStore(msg.Info.Name, store)
-		}
+		acct.dirlist.Update(msg)
 	case *types.DirectoryContents:
 		if store, ok := acct.dirlist.SelectedMsgStore(); ok {
 			if acct.msglist.Store() == nil {
 				acct.msglist.SetStore(store)
 			}
 			store.Update(msg)
-			acct.SetStatus(statusline.Threading(store.ThreadedView()))
+			acct.SetStatus(state.Threading(store.ThreadedView()))
 		}
 		if acct.newConn && len(msg.Uids) == 0 {
 			acct.checkMailOnStartup()
@@ -314,7 +328,7 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 				acct.msglist.SetStore(store)
 			}
 			store.Update(msg)
-			acct.SetStatus(statusline.Threading(store.ThreadedView()))
+			acct.SetStatus(state.Threading(store.ThreadedView()))
 		}
 		if acct.newConn && len(msg.Threads) == 0 {
 			acct.checkMailOnStartup()
@@ -328,10 +342,10 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 			store.Update(msg)
 		}
 	case *types.MessagesDeleted:
+		if dir := acct.dirlist.SelectedDirectory(); dir != nil {
+			dir.Exists -= len(msg.Uids)
+		}
 		if store, ok := acct.dirlist.SelectedMsgStore(); ok {
-			store.DirInfo.Exists -= len(msg.Uids)
-			// False to trigger recount of recent/unseen
-			store.DirInfo.AccurateCounts = false
 			store.Update(msg)
 		}
 	case *types.MessagesCopied:
@@ -342,7 +356,7 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 		acct.labels = msg.Labels
 	case *types.ConnError:
 		log.Errorf("[%s] connection error: %v", acct.acct.Name, msg.Error)
-		acct.SetStatus(statusline.SetConnected(false))
+		acct.SetStatus(state.SetConnected(false))
 		acct.PushError(msg.Error)
 		acct.msglist.SetStore(nil)
 		acct.worker.PostAction(&types.Reconnect{}, nil)
@@ -351,11 +365,12 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 		acct.PushError(msg.Error)
 	}
 	acct.UpdateStatus()
+	acct.setTitle()
 }
 
 func (acct *AccountView) updateDirCounts(destination string, uids []uint32) {
-	// Only update the destination destStore if it is initialized
-	if destStore, ok := acct.dirlist.MsgStore(destination); ok {
+	// Only update the destination destDir if it is initialized
+	if destDir := acct.dirlist.Directory(destination); destDir != nil {
 		var recent, unseen int
 		var accurate bool = true
 		for _, uid := range uids {
@@ -369,31 +384,20 @@ func (acct *AccountView) updateDirCounts(destination string, uids []uint32) {
 				accurate = false
 				break
 			}
-			seen := false
-			for _, flag := range msg.Flags {
-				if flag == models.SeenFlag {
-					seen = true
-				}
-				if flag == models.RecentFlag {
-					recent++
-				}
+			seen := msg.Flags.Has(models.SeenFlag)
+			if msg.Flags.Has(models.RecentFlag) {
+				recent++
 			}
 			if !seen {
 				unseen++
 			}
 		}
 		if accurate {
-			destStore.DirInfo.Recent += recent
-			destStore.DirInfo.Unseen += unseen
-			destStore.DirInfo.Exists += len(uids)
-			// True. For imap, we don't have the message in the store until we
-			// Select so we need to rely on the math we just did for accurate
-			// counts
-			destStore.DirInfo.AccurateCounts = true
+			destDir.Recent += recent
+			destDir.Unseen += unseen
+			destDir.Exists += len(uids)
 		} else {
-			destStore.DirInfo.Exists += len(uids)
-			// False to trigger recount of recent/unseen
-			destStore.DirInfo.AccurateCounts = false
+			destDir.Exists += len(uids)
 		}
 	}
 }
@@ -422,7 +426,7 @@ func (acct *AccountView) CheckMail() {
 	dirs = acct.dirlist.FilterDirs(dirs, acct.AccountConfig().CheckMailInclude, false)
 	dirs = acct.dirlist.FilterDirs(dirs, exclude, true)
 	log.Debugf("Checking for new mail on account %s", acct.Name())
-	acct.SetStatus(statusline.ConnectionActivity("Checking for new mail..."))
+	acct.SetStatus(state.ConnectionActivity("Checking for new mail..."))
 	msg := &types.CheckMail{
 		Directories: dirs,
 		Command:     acct.acct.CheckMailCmd,
@@ -441,7 +445,7 @@ func (acct *AccountView) CheckMail() {
 			}
 			acct.worker.PostAction(checkMailMsg, cb)
 		} else { // Done
-			acct.SetStatus(statusline.ConnectionActivity(""))
+			acct.SetStatus(state.ConnectionActivity(""))
 			acct.Lock()
 			acct.checkingMail = false
 			acct.Unlock()
@@ -468,8 +472,9 @@ func (acct *AccountView) checkMailOnStartup() {
 func (acct *AccountView) CheckMailTimer(d time.Duration) {
 	acct.ticker = time.NewTicker(d)
 	go func() {
+		defer log.PanicHandler()
 		for range acct.ticker.C {
-			if !acct.state.Connected() {
+			if !acct.state.Connected {
 				continue
 			}
 			acct.CheckMail()
@@ -592,3 +597,23 @@ func (acct *AccountView) Vsplit(n int) error {
 	acct.updateSplitView(acct.msglist.Selected())
 	return nil
 }
+
+// setTitle executes the title template and sets the tab title
+func (acct *AccountView) setTitle() {
+	var data state.TemplateData
+
+	if acct.tab == nil {
+		return
+	}
+	data.SetAccount(acct.acct)
+	data.SetFolder(acct.Directories().SelectedDirectory())
+	data.SetRUE(acct.dirlist.List(), acct.dirlist.GetRUECount)
+
+	var buf bytes.Buffer
+	err := templates.Render(acct.uiConf.TabTitleAccount, &buf, &data)
+	if err != nil {
+		acct.PushError(err)
+		return
+	}
+	acct.tab.SetTitle(buf.String())
+}
