@@ -26,7 +26,7 @@ import (
 
 type Aerc struct {
 	accounts    map[string]*AccountView
-	cmd         func(cmd []string) error
+	cmd         func([]string, *config.AccountConfig, *models.MessageInfo) error
 	cmdHistory  lib.History
 	complete    func(cmd string) []string
 	focused     ui.Interactive
@@ -52,14 +52,15 @@ type Choice struct {
 }
 
 func NewAerc(
-	crypto crypto.Provider, cmd func(cmd []string) error,
+	crypto crypto.Provider,
+	cmd func([]string, *config.AccountConfig, *models.MessageInfo) error,
 	complete func(cmd string) []string, cmdHistory lib.History,
 	deferLoop chan struct{},
 ) *Aerc {
 	tabs := ui.NewTabs(config.Ui)
 
 	statusbar := ui.NewStack(config.Ui)
-	statusline := NewStatusLine(config.Ui)
+	statusline := &StatusLine{}
 	statusbar.Push(statusline)
 
 	grid := ui.NewGrid().Rows([]ui.GridSpec{
@@ -87,7 +88,6 @@ func NewAerc(
 	}
 
 	statusline.SetAerc(aerc)
-	config.Triggers.ExecuteCommand = cmd
 
 	for _, acct := range config.Accounts {
 		view, err := NewAccountView(aerc, acct, aerc, deferLoop)
@@ -95,7 +95,7 @@ func NewAerc(
 			tabs.Add(errorScreen(err.Error()), acct.Name, nil)
 		} else {
 			aerc.accounts[acct.Name] = view
-			tabs.Add(view, acct.Name, view.UiConfig())
+			view.tab = tabs.Add(view, acct.Name, view.UiConfig())
 		}
 	}
 
@@ -117,19 +117,39 @@ func NewAerc(
 			return
 		case *AccountWizard:
 			return
-		case *Composer:
-			aerc.RemoveTab(content)
-			content.Close()
-		case *Terminal:
-			content.Close(nil)
-		case *MessageViewer:
-			aerc.RemoveTab(content)
+		default:
+			aerc.RemoveTab(content, true)
 		}
 	}
 
+	aerc.showConfigWarnings()
+
 	return aerc
 }
 
+func (aerc *Aerc) showConfigWarnings() {
+	var dialogs []ui.DrawableInteractive
+
+	callback := func(string, error) {
+		aerc.CloseDialog()
+		if len(dialogs) > 0 {
+			d := dialogs[0]
+			dialogs = dialogs[1:]
+			aerc.AddDialog(d)
+		}
+	}
+
+	for _, w := range config.Warnings {
+		dialogs = append(dialogs, NewSelectorDialog(
+			w.Title, w.Body, []string{"OK"}, 0,
+			aerc.SelectedAccountUiConfig(),
+			callback,
+		))
+	}
+
+	callback("", nil)
+}
+
 func (aerc *Aerc) OnBeep(f func() error) {
 	aerc.beep = f
 }
@@ -460,13 +480,19 @@ func (aerc *Aerc) NewTab(clickable ui.Drawable, name string) *ui.Tab {
 	return tab
 }
 
-func (aerc *Aerc) RemoveTab(tab ui.Drawable) {
+func (aerc *Aerc) RemoveTab(tab ui.Drawable, closeContent bool) {
 	aerc.tabs.Remove(tab)
 	aerc.UpdateStatus()
+	if content, ok := tab.(ui.Closeable); ok && closeContent {
+		content.Close()
+	}
 }
 
-func (aerc *Aerc) ReplaceTab(tabSrc ui.Drawable, tabTarget ui.Drawable, name string) {
+func (aerc *Aerc) ReplaceTab(tabSrc ui.Drawable, tabTarget ui.Drawable, name string, closeSrc bool) {
 	aerc.tabs.Replace(tabSrc, tabTarget, name)
+	if content, ok := tabSrc.(ui.Closeable); ok && closeSrc {
+		content.Close()
+	}
 }
 
 func (aerc *Aerc) MoveTab(i int, relative bool) {
@@ -513,24 +539,16 @@ func (aerc *Aerc) SelectPreviousTab() bool {
 	return aerc.tabs.SelectPrevious()
 }
 
-func (aerc *Aerc) SetStatus(status string) *StatusMessage {
-	return aerc.statusline.Set(status)
-}
-
 func (aerc *Aerc) UpdateStatus() {
 	if acct := aerc.SelectedAccount(); acct != nil {
-		acct.UpdateStatus()
+		aerc.statusline.Update(acct)
 	} else {
-		aerc.ClearStatus()
+		aerc.statusline.Clear()
 	}
 }
 
-func (aerc *Aerc) ClearStatus() {
-	aerc.statusline.Set("")
-}
-
-func (aerc *Aerc) SetError(status string) *StatusMessage {
-	return aerc.statusline.SetError(status)
+func (aerc *Aerc) SetError(err string) {
+	aerc.statusline.SetError(err)
 }
 
 func (aerc *Aerc) PushStatus(text string, expiry time.Duration) *StatusMessage {
@@ -584,7 +602,7 @@ func (aerc *Aerc) BeginExCommand(cmd string) {
 		if err != nil {
 			aerc.PushError(err.Error())
 		}
-		err = aerc.cmd(parts)
+		err = aerc.cmd(parts, nil, nil)
 		if err != nil {
 			aerc.PushError(err.Error())
 		}
@@ -610,7 +628,7 @@ func (aerc *Aerc) RegisterPrompt(prompt string, cmd []string) {
 		if text != "" {
 			cmd = append(cmd, text)
 		}
-		err := aerc.cmd(cmd)
+		err := aerc.cmd(cmd, nil, nil)
 		if err != nil {
 			aerc.PushError(err.Error())
 		}
@@ -637,7 +655,7 @@ func (aerc *Aerc) RegisterChoices(choices []Choice) {
 		if !ok {
 			return
 		}
-		err := aerc.cmd(cmd)
+		err := aerc.cmd(cmd, nil, nil)
 		if err != nil {
 			aerc.PushError(err.Error())
 		}
@@ -658,6 +676,7 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 		return fmt.Errorf("Could not parse to: %w", err)
 	}
 	h.SetAddressList("to", to)
+	template := config.Templates.NewMessage
 	for key, vals := range addr.Query() {
 		switch strings.ToLower(key) {
 		case "account":
@@ -687,6 +706,9 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 		case "subject":
 			subject = strings.Join(vals, ",")
 			h.SetText("Subject", subject)
+		case "template":
+			template = strings.Join(vals, "")
+			log.Tracef("template set to %s", template)
 		case "attach":
 			for _, path := range vals {
 				// remove a potential file:// prefix.
@@ -709,10 +731,13 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 		return errors.New("No account selected")
 	}
 
+	defer ui.Invalidate()
+	defer ui.QueueRedraw()
+
 	composer, err := NewComposer(aerc, acct,
-		acct.AccountConfig(), acct.Worker(), "", h, models.OriginalMail{})
+		acct.AccountConfig(), acct.Worker(), template, h, nil)
 	if err != nil {
-		return nil
+		return err
 	}
 	composer.SetContents(strings.NewReader(body))
 	composer.FocusEditor("subject")
@@ -724,15 +749,7 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 	if to == nil {
 		composer.FocusEditor("to")
 	}
-	tab := aerc.NewTab(composer, title)
-	composer.OnHeaderChange("Subject", func(subject string) {
-		if subject == "" {
-			tab.Name = "New email"
-		} else {
-			tab.Name = subject
-		}
-		ui.Invalidate()
-	})
+	composer.Tab = aerc.NewTab(composer, title)
 
 	for _, file := range attachments {
 		composer.AddAttachment(file)
@@ -748,7 +765,7 @@ func (aerc *Aerc) Mbox(source string) error {
 		aerc.PushStatus(info, 10*time.Second)
 		log.Debugf(info)
 	} else {
-		acctConf.From = "<user@localhost>"
+		acctConf.From = &mail.Address{Address: "user@localhost"}
 	}
 	acctConf.Name = "mbox"
 	acctConf.Source = source
@@ -757,6 +774,9 @@ func (aerc *Aerc) Mbox(source string) error {
 	acctConf.Postpone = "Drafts"
 	acctConf.CopyTo = "Sent"
 
+	defer ui.Invalidate()
+	defer ui.QueueRedraw()
+
 	mboxView, err := NewAccountView(aerc, &acctConf, aerc, nil)
 	if err != nil {
 		aerc.NewTab(errorScreen(err.Error()), acctConf.Name)
@@ -767,6 +787,12 @@ func (aerc *Aerc) Mbox(source string) error {
 	return nil
 }
 
+func (aerc *Aerc) Command(args []string) error {
+	defer ui.QueueRedraw()
+	defer ui.Invalidate()
+	return aerc.cmd(args, nil, nil)
+}
+
 func (aerc *Aerc) CloseBackends() error {
 	var returnErr error
 	for _, acct := range aerc.accounts {
