@@ -1,6 +1,7 @@
 package widgets
 
 import (
+	"bufio"
 	"bytes"
 	"fmt"
 	"io"
@@ -22,6 +23,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/lib/state"
 	"git.sr.ht/~rjarry/aerc/lib/templates"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/log"
@@ -33,7 +35,7 @@ type Composer struct {
 	sync.Mutex
 	editors map[string]*headerEditor // indexes in lower case (from / cc / bcc)
 	header  *mail.Header
-	parent  models.OriginalMail // parent of current message, only set if reply
+	parent  *models.OriginalMail // parent of current message, only set if reply
 
 	acctConfig *config.AccountConfig
 	acct       *AccountView
@@ -56,18 +58,20 @@ type Composer struct {
 	focusable []ui.MouseableDrawableInteractive
 	focused   int
 	sent      bool
+	archive   string
 
 	onClose []func(ti *Composer)
 
 	width int
 
 	textParts []*lib.Part
+	Tab       *ui.Tab
 }
 
 func NewComposer(
 	aerc *Aerc, acct *AccountView, acctConfig *config.AccountConfig,
 	worker *types.Worker, template string,
-	h *mail.Header, orig models.OriginalMail,
+	h *mail.Header, orig *models.OriginalMail,
 ) (*Composer, error) {
 	if h == nil {
 		h = new(mail.Header)
@@ -92,8 +96,11 @@ func NewComposer(
 		completer: nil,
 	}
 
-	templateData := templates.ParseTemplateData(h, orig)
-	if err := c.AddTemplate(template, templateData); err != nil {
+	var data state.TemplateData
+	data.SetAccount(acct.acct)
+	data.SetFolder(acct.Directories().SelectedDirectory())
+	data.SetHeaders(h, orig)
+	if err := c.AddTemplate(template, &data); err != nil {
 		return nil, err
 	}
 	c.AddSignature()
@@ -127,7 +134,7 @@ func (c *Composer) SwitchAccount(newAcct *AccountView) error {
 		editor.loadValue()
 	}
 	c.Invalidate()
-	log.Debugf("account sucessfully switched")
+	log.Debugf("account successfully switched")
 	return nil
 }
 
@@ -137,16 +144,10 @@ func (c *Composer) setupFor(view *AccountView) error {
 	// set new account
 	c.acct = view
 	c.worker = view.Worker()
-
+	c.acctConfig = c.acct.AccountConfig()
 	// Set from header if not already in header
 	if fl, err := c.header.AddressList("from"); err != nil || fl == nil {
-		fl, err = mail.ParseAddressList(view.acct.From)
-		if err != nil {
-			return err
-		}
-		if fl != nil {
-			c.header.SetAddressList("from", fl)
-		}
+		c.header.SetAddressList("from", []*mail.Address{view.acct.From})
 	}
 
 	// update completer
@@ -227,6 +228,15 @@ func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
 			default:
 				c.focusable = append(c.focusable, e)
 			}
+			e.OnChange(func() {
+				c.setTitle()
+				ui.Invalidate()
+			})
+			e.OnFocusLost(func() {
+				c.PrepareHeader() //nolint:errcheck // tab title only, fine if it's not valid yet
+				c.setTitle()
+				ui.Invalidate()
+			})
 		}
 	}
 
@@ -255,14 +265,19 @@ func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
 	}
 }
 
-func (c *Composer) SetSent() {
+func (c *Composer) SetSent(archive string) {
 	c.sent = true
+	c.archive = archive
 }
 
 func (c *Composer) Sent() bool {
 	return c.sent
 }
 
+func (c *Composer) Archive() string {
+	return c.archive
+}
+
 func (c *Composer) SetAttachKey(attach bool) error {
 	if !attach {
 		name := c.crypto.signKey + ".asc"
@@ -288,10 +303,7 @@ func (c *Composer) SetAttachKey(attach bool) error {
 			if c.acctConfig.PgpKeyId != "" {
 				s = c.acctConfig.PgpKeyId
 			} else {
-				s, err = getSenderEmail(c)
-				if err != nil {
-					return err
-				}
+				s = c.acctConfig.From.Address
 			}
 			c.crypto.signKey, err = c.aerc.Crypto.GetSignerKeyId(s)
 			if err != nil {
@@ -376,18 +388,11 @@ func (c *Composer) updateCrypto() error {
 		uiConfig := c.acct.UiConfig()
 		c.crypto = newCryptoStatus(uiConfig)
 	}
-	var err error
-	// Check if signKey is empty so we only run this once
-	if c.sign && c.crypto.signKey == "" {
+	if c.sign {
 		cp := c.aerc.Crypto
-		var s string
-		if c.acctConfig.PgpKeyId != "" {
-			s = c.acctConfig.PgpKeyId
-		} else {
-			s, err = getSenderEmail(c)
-			if err != nil {
-				return err
-			}
+		s, err := c.Signer()
+		if err != nil {
+			return errors.Wrap(err, "Signer")
 		}
 		c.crypto.signKey, err = cp.GetSignerKeyId(s)
 		if err != nil {
@@ -491,7 +496,7 @@ func (c *Composer) RemovePart(mimetype string) error {
 	return fmt.Errorf("%s part not found", mimetype)
 }
 
-func (c *Composer) AddTemplate(template string, data interface{}) error {
+func (c *Composer) AddTemplate(template string, data models.TemplateData) error {
 	if template == "" {
 		return nil
 	}
@@ -533,6 +538,10 @@ func (c *Composer) AddSignature() {
 	} else {
 		signature = c.readSignatureFromFile()
 	}
+	if len(bytes.TrimSpace(signature)) == 0 {
+		return
+	}
+	signature = ensureSignatureDelimiter(signature)
 	c.AppendContents(bytes.NewReader(signature))
 }
 
@@ -564,6 +573,21 @@ func (c *Composer) readSignatureFromFile() []byte {
 	return signature
 }
 
+func ensureSignatureDelimiter(signature []byte) []byte {
+	buf := bytes.NewBuffer(signature)
+	scanner := bufio.NewScanner(buf)
+	for scanner.Scan() {
+		line := scanner.Text()
+		if line == "-- " {
+			// signature contains standard delimiter, we're good
+			return signature
+		}
+	}
+	// signature does not contain standard delimiter, prepend one
+	sig := "\n\n-- \n" + strings.TrimLeft(string(signature), " \t\r\n")
+	return []byte(sig)
+}
+
 func (c *Composer) FocusTerminal() *Composer {
 	c.Lock()
 	defer c.Unlock()
@@ -599,6 +623,7 @@ func (c *Composer) OnClose(fn func(composer *Composer)) {
 }
 
 func (c *Composer) Draw(ctx *ui.Context) {
+	c.setTitle()
 	c.width = ctx.Width()
 	c.grid.Draw(ctx)
 }
@@ -674,6 +699,14 @@ func (c *Composer) Focus(focus bool) {
 	c.Unlock()
 }
 
+func (c *Composer) Show(visible bool) {
+	c.Lock()
+	if vis, ok := c.focusable[c.focused].(ui.Visible); ok {
+		vis.Show(visible)
+	}
+	c.Unlock()
+}
+
 func (c *Composer) Config() *config.AccountConfig {
 	return c.acctConfig
 }
@@ -711,18 +744,6 @@ func (c *Composer) PrepareHeader() (*mail.Header, error) {
 	return c.header, nil
 }
 
-func getSenderEmail(c *Composer) (string, error) {
-	// add the from: field also to the 'recipients' list
-	if c.acctConfig.From == "" {
-		return "", errors.New("No 'From' configured for this account")
-	}
-	from, err := mail.ParseAddress(c.acctConfig.From)
-	if err != nil {
-		return "", errors.Wrap(err, "ParseAddress(config.From)")
-	}
-	return from.Address, nil
-}
-
 func getRecipientsEmail(c *Composer) ([]string, error) {
 	h, err := c.PrepareHeader()
 	if err != nil {
@@ -751,6 +772,30 @@ func getRecipientsEmail(c *Composer) ([]string, error) {
 	return results, nil
 }
 
+func (c *Composer) Signer() (string, error) {
+	signer := ""
+
+	if c.acctConfig.PgpKeyId != "" {
+		// get key from explicitly set keyid
+		signer = c.acctConfig.PgpKeyId
+	} else {
+		// get signer from `from` header
+		from, err := c.header.AddressList("from")
+		if err != nil {
+			return "", err
+		}
+
+		if len(from) > 0 {
+			signer = from[0].Address
+		} else {
+			// fall back to address from config
+			signer = c.acctConfig.From.Address
+		}
+	}
+
+	return signer, nil
+}
+
 func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 	if err := c.reloadEmail(); err != nil {
 		return err
@@ -767,13 +812,9 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 
 		signer := ""
 		if c.sign {
-			if c.acctConfig.PgpKeyId != "" {
-				signer = c.acctConfig.PgpKeyId
-			} else {
-				signer, err = getSenderEmail(c)
-				if err != nil {
-					return err
-				}
+			signer, err = c.Signer()
+			if err != nil {
+				return errors.Wrap(err, "Signer")
 			}
 		}
 
@@ -833,20 +874,20 @@ func (c *Composer) ShouldWarnAttachment() (bool, error) {
 }
 
 func writeMsgImpl(c *Composer, header *mail.Header, writer io.Writer) error {
+	mimeParams := map[string]string{"Charset": "UTF-8"}
+	if config.Compose.FormatFlowed {
+		mimeParams["Format"] = "Flowed"
+	}
 	if len(c.attachments) == 0 && len(c.textParts) == 0 {
 		// no attachments
-		return writeInlineBody(header, c.email, writer)
+		return writeInlineBody(header, c.email, writer, mimeParams)
 	} else {
 		// with attachments
 		w, err := mail.CreateWriter(writer, *header)
 		if err != nil {
 			return errors.Wrap(err, "CreateWriter")
 		}
-		newPart, err := lib.NewPart(
-			"text/plain",
-			map[string]string{"Charset": "UTF-8"},
-			c.email,
-		)
+		newPart, err := lib.NewPart("text/plain", mimeParams, c.email)
 		if err != nil {
 			return err
 		}
@@ -864,8 +905,13 @@ func writeMsgImpl(c *Composer, header *mail.Header, writer io.Writer) error {
 	return nil
 }
 
-func writeInlineBody(header *mail.Header, body io.Reader, writer io.Writer) error {
-	header.SetContentType("text/plain", map[string]string{"charset": "UTF-8"})
+func writeInlineBody(
+	header *mail.Header,
+	body io.Reader,
+	writer io.Writer,
+	mimeParams map[string]string,
+) error {
+	header.SetContentType("text/plain", mimeParams)
 	w, err := mail.CreateSingleInlineWriter(writer, *header)
 	if err != nil {
 		return errors.Wrap(err, "CreateSingleInlineWriter")
@@ -1491,18 +1537,65 @@ func (c *Composer) checkEncryptionKeys(_ string) bool {
 			mk = append(mk, rcpt)
 		}
 	}
-	if len(mk) > 0 {
+
+	encrypt := true
+	switch {
+	case len(mk) > 0:
 		c.SetEncrypt(false)
 		st := fmt.Sprintf("Cannot encrypt, missing keys: %s", strings.Join(mk, ", "))
+		if c.Config().PgpOpportunisticEncrypt {
+			switch c.Config().PgpErrorLevel {
+			case config.PgpErrorLevelWarn:
+				c.aerc.statusline.PushWarning(st)
+				return false
+			case config.PgpErrorLevelNone:
+				return false
+			case config.PgpErrorLevelError:
+				// Continue to the default
+			}
+		}
 		c.aerc.statusline.PushError(st)
-		return false
+		encrypt = false
+	case len(rcpts) == 0:
+		encrypt = false
 	}
+
 	// If callbacks were registered, encrypt will be set when user removes
 	// recipients with missing keys
-	c.encrypt = true
+	c.encrypt = encrypt
 	err = c.updateCrypto()
 	if err != nil {
 		log.Warnf("failed update crypto: %v", err)
 	}
 	return true
 }
+
+// setTitle executes the title template and sets the tab title
+func (c *Composer) setTitle() {
+	if c.Tab == nil {
+		return
+	}
+
+	var data state.TemplateData
+
+	header := c.header.Copy()
+	// Get subject direct from the textinput
+	subject, ok := c.editors["subject"]
+	if ok {
+		header.SetSubject(subject.input.String())
+	}
+	if header.Get("subject") == "" {
+		header.SetSubject("New Email")
+	}
+	data.SetAccount(c.acctConfig)
+	data.SetFolder(c.acct.Directories().SelectedDirectory())
+	data.SetHeaders(&header, c.parent)
+
+	var buf bytes.Buffer
+	err := templates.Render(c.acct.UiConfig().TabTitleComposer, &buf, &data)
+	if err != nil {
+		c.acct.PushError(err)
+		return
+	}
+	c.Tab.SetTitle(buf.String())
+}
