@@ -1,6 +1,7 @@
 package widgets
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
 	"io"
@@ -10,6 +11,7 @@ import (
 	"sync/atomic"
 
 	"github.com/danwakefield/fnmatch"
+	"github.com/emersion/go-message/textproto"
 	"github.com/gdamore/tcell/v2"
 	"github.com/google/shlex"
 	"github.com/mattn/go-runewidth"
@@ -38,7 +40,6 @@ type MessageViewer struct {
 type PartSwitcher struct {
 	parts          []*PartViewer
 	selected       int
-	showHeaders    bool
 	alwaysShowMime bool
 
 	height int
@@ -222,7 +223,6 @@ func createSwitcher(
 ) error {
 	var err error
 	switcher.selected = -1
-	switcher.showHeaders = config.Viewer.ShowHeaders
 	switcher.alwaysShowMime = config.Viewer.AlwaysShowMime
 
 	if msg.MessageInfo().Error != nil {
@@ -390,11 +390,10 @@ func (mv *MessageViewer) Bindings() string {
 	}
 }
 
-func (mv *MessageViewer) Close() error {
+func (mv *MessageViewer) Close() {
 	if mv.switcher != nil {
 		mv.switcher.Cleanup()
 	}
-	return nil
 }
 
 func (ps *PartSwitcher) Invalidate() {
@@ -407,6 +406,12 @@ func (ps *PartSwitcher) Focus(focus bool) {
 	}
 }
 
+func (ps *PartSwitcher) Show(visible bool) {
+	if ps.parts[ps.selected].term != nil {
+		ps.parts[ps.selected].term.Show(visible)
+	}
+}
+
 func (ps *PartSwitcher) Event(event tcell.Event) bool {
 	return ps.parts[ps.selected].Event(event)
 }
@@ -506,22 +511,25 @@ func (mv *MessageViewer) Focus(focus bool) {
 	mv.switcher.Focus(focus)
 }
 
+func (mv *MessageViewer) Show(visible bool) {
+	mv.switcher.Show(visible)
+}
+
 type PartViewer struct {
-	acctConfig  *config.AccountConfig
-	err         error
-	fetched     bool
-	filter      *exec.Cmd
-	index       []int
-	msg         lib.MessageView
-	pager       *exec.Cmd
-	pagerin     io.WriteCloser
-	part        *models.BodyStructure
-	showHeaders bool
-	source      io.Reader
-	term        *Terminal
-	grid        *ui.Grid
-	uiConfig    *config.UIConfig
-	copying     int32
+	acctConfig *config.AccountConfig
+	err        error
+	fetched    bool
+	filter     *exec.Cmd
+	index      []int
+	msg        lib.MessageView
+	pager      *exec.Cmd
+	pagerin    io.WriteCloser
+	part       *models.BodyStructure
+	source     io.Reader
+	term       *Terminal
+	grid       *ui.Grid
+	uiConfig   *config.UIConfig
+	copying    int32
 
 	links []string
 }
@@ -605,6 +613,8 @@ func NewPartViewer(
 			fmt.Sprintf("AERC_SUBJECT=%s", info.Envelope.Subject))
 		filter.Env = append(filter.Env, fmt.Sprintf("AERC_FROM=%s",
 			format.FormatAddresses(info.Envelope.From)))
+		filter.Env = append(filter.Env, fmt.Sprintf("AERC_STYLESET=%s",
+			acct.UiConfig().StyleSetPath()))
 		log.Debugf("<%s> part=%v %s: %v | %v",
 			info.Envelope.MessageId, curindex, mime, filter, pager)
 		if pagerin, err = pager.StdinPipe(); err != nil {
@@ -626,17 +636,16 @@ func NewPartViewer(
 	copy(index, curindex)
 
 	pv := &PartViewer{
-		acctConfig:  acct.AccountConfig(),
-		filter:      filter,
-		index:       index,
-		msg:         msg,
-		pager:       pager,
-		pagerin:     pagerin,
-		part:        part,
-		showHeaders: config.Viewer.ShowHeaders,
-		term:        term,
-		grid:        grid,
-		uiConfig:    acct.UiConfig(),
+		acctConfig: acct.AccountConfig(),
+		filter:     filter,
+		index:      index,
+		msg:        msg,
+		pager:      pager,
+		pagerin:    pagerin,
+		part:       part,
+		term:       term,
+		grid:       grid,
+		uiConfig:   acct.UiConfig(),
 	}
 
 	if term != nil {
@@ -690,36 +699,67 @@ func (pv *PartViewer) attemptCopy() {
 
 func (pv *PartViewer) writeMailHeaders() {
 	info := pv.msg.MessageInfo()
-	if pv.showHeaders && info.RFC822Headers != nil {
-		// header need to bypass the filter, else we run into issues
-		// with the filter messing with newlines etc.
-		// hence all writes in this block go directly to the pager
-		fields := info.RFC822Headers.Fields()
-		for fields.Next() {
-			var value string
-			var err error
-			if value, err = fields.Text(); err != nil {
-				// better than nothing, use the non decoded version
-				value = fields.Value()
+	if config.Viewer.ShowHeaders && info.RFC822Headers != nil {
+		var file io.WriteCloser
+
+		for _, f := range config.Filters {
+			if f.Type != config.FILTER_HEADERS {
+				continue
 			}
-			field := fmt.Sprintf(
-				"%s: %s\n", fields.Key(), value)
-			_, err = pv.pagerin.Write([]byte(field))
-			if err != nil {
-				log.Errorf("failed to write to stdin of pager: %v", err)
+			log.Debugf("<%s> piping headers in filter: %s",
+				info.Envelope.MessageId, f.Command)
+			filter := exec.Command("sh", "-c", f.Command)
+			if pv.filter != nil {
+				// inherit from filter env
+				filter.Env = pv.filter.Env
 			}
+
+			stdin, err := filter.StdinPipe()
+			if err == nil {
+				filter.Stdout = pv.pagerin
+				filter.Stderr = pv.pagerin
+				err := filter.Start()
+				if err == nil {
+					//nolint:errcheck // who cares?
+					defer filter.Wait()
+					file = stdin
+				} else {
+					log.Errorf(
+						"failed to start header filter: %v",
+						err)
+				}
+			} else {
+				log.Errorf("failed to create pipe: %v", err)
+			}
+			break
 		}
+		if file == nil {
+			file = pv.pagerin
+		} else {
+			defer file.Close()
+		}
+
+		var buf bytes.Buffer
+		err := textproto.WriteHeader(&buf, info.RFC822Headers.Header.Header)
+		if err != nil {
+			log.Errorf("failed to format headers: %v", err)
+		}
+		_, err = file.Write(bytes.TrimRight(buf.Bytes(), "\r\n"))
+		if err != nil {
+			log.Errorf("failed to write headers: %v", err)
+		}
+
 		// virtual header
 		if len(info.Labels) != 0 {
 			labels := fmtHeader(info, "Labels", "", "", "", "")
-			_, err := pv.pagerin.Write([]byte(fmt.Sprintf("Labels: %s\n", labels)))
+			_, err := file.Write([]byte(fmt.Sprintf("\r\nLabels: %s", labels)))
 			if err != nil {
-				log.Errorf("failed to write to stdin of pager: %v", err)
+				log.Errorf("failed to write to labels: %v", err)
 			}
 		}
-		_, err := pv.pagerin.Write([]byte{'\n'})
+		_, err = file.Write([]byte{'\r', '\n', '\r', '\n'})
 		if err != nil {
-			log.Errorf("failed to write to stdin of pager: %v", err)
+			log.Errorf("failed to write empty line: %v", err)
 		}
 	}
 }
@@ -816,7 +856,7 @@ func (pv *PartViewer) Draw(ctx *ui.Context) {
 
 func (pv *PartViewer) Cleanup() {
 	if pv.term != nil {
-		pv.term.Close(nil)
+		pv.term.Close()
 	}
 }
 
