@@ -4,11 +4,13 @@ import (
 	"bufio"
 	"bytes"
 	"encoding/gob"
+	"errors"
 	"fmt"
 	"os"
 	"path"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/lib/parse"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -24,10 +26,19 @@ type CachedHeader struct {
 	Envelope      models.Envelope
 	InternalDate  time.Time
 	Uid           uint32
+	Size          uint32
 	Header        []byte
 	Created       time.Time
 }
 
+var (
+	// cacheTag should be updated when changing the cache
+	// structure; this will ensure that the user's cache is cleared and
+	// reloaded when the underlying cache structure changes
+	cacheTag    = []byte("0001")
+	cacheTagKey = []byte("cache.tag")
+)
+
 // initCacheDb opens (or creates) the database for the cache. One database is
 // created per account
 func (w *IMAPWorker) initCacheDb(acct string) {
@@ -46,8 +57,26 @@ func (w *IMAPWorker) initCacheDb(acct string) {
 	}
 	w.cache = db
 	log.Debugf("cache db opened: %s", p)
-	if w.config.cacheMaxAge.Hours() > 0 {
-		go w.cleanCache(p)
+
+	tag, err := w.cache.Get(cacheTagKey, nil)
+	clearCache := errors.Is(err, leveldb.ErrNotFound) ||
+		!bytes.Equal(tag, cacheTag)
+	switch {
+	case clearCache:
+		log.Infof("current cache tag is '%s' but found '%s'",
+			cacheTag, tag)
+		log.Warnf("tag mismatch: clear cache")
+		w.clearCache()
+		if err = w.cache.Put(cacheTagKey, cacheTag, nil); err != nil {
+			log.Errorf("could not set the current cache tag")
+		}
+	case err != nil:
+		log.Errorf("could not get the cache tag from db")
+	default:
+		log.Tracef("cache version match")
+		if w.config.cacheMaxAge.Hours() > 0 {
+			go w.cleanCache(p)
+		}
 	}
 }
 
@@ -66,6 +95,7 @@ func (w *IMAPWorker) cacheHeader(mi *models.MessageInfo) {
 		Envelope:      *mi.Envelope,
 		InternalDate:  mi.InternalDate,
 		Uid:           mi.Uid,
+		Size:          mi.Size,
 		Header:        hdr.Bytes(),
 		Created:       time.Now(),
 	}
@@ -114,13 +144,11 @@ func (w *IMAPWorker) getCachedHeaders(msg *types.FetchMessageHeaders) []uint32 {
 		mi := &models.MessageInfo{
 			BodyStructure: &ch.BodyStructure,
 			Envelope:      &ch.Envelope,
-			Flags:         []models.Flag{models.SeenFlag}, // Always return a SEEN flag
+			Flags:         models.SeenFlag, // Always return a SEEN flag
 			Uid:           ch.Uid,
 			RFC822Headers: hdr,
-		}
-		refs, err := hdr.MsgIDList("references")
-		if err != nil {
-			mi.Refs = refs
+			Refs:          parse.MsgIDList(hdr, "references"),
+			Size:          ch.Size,
 		}
 		log.Tracef("located cached header %s.%s", uv, u)
 		w.worker.PostMessage(&types.MessageInfo{
@@ -145,10 +173,14 @@ func cacheDir() (string, error) {
 
 // cleanCache removes stale entries from the selected mailbox cachedb
 func (w *IMAPWorker) cleanCache(path string) {
+	defer log.PanicHandler()
 	start := time.Now()
 	var scanned, removed int
 	iter := w.cache.NewIterator(nil, nil)
 	for iter.Next() {
+		if bytes.Equal(iter.Key(), cacheTagKey) {
+			continue
+		}
 		data := iter.Value()
 		ch := &CachedHeader{}
 		dec := gob.NewDecoder(bytes.NewReader(data))
@@ -173,3 +205,14 @@ func (w *IMAPWorker) cleanCache(path string) {
 	log.Debugf("%s: removed %d/%d expired entries in %s",
 		path, removed, scanned, elapsed)
 }
+
+// clearCache clears the entire cache
+func (w *IMAPWorker) clearCache() {
+	iter := w.cache.NewIterator(nil, nil)
+	for iter.Next() {
+		if err := w.cache.Delete(iter.Key(), nil); err != nil {
+			log.Errorf("error clearing cache: %v", err)
+		}
+	}
+	iter.Release()
+}
