@@ -82,7 +82,7 @@ func (w *IMAPWorker) connect() (*client.Client, error) {
 			}
 		} else if w.config.xoauth2.Enabled {
 			if err := w.config.xoauth2.Authenticate(
-				username, password, c); err != nil {
+				username, password, w.worker.Name, c); err != nil {
 				return nil, err
 			}
 		} else if err := c.Login(username, password); err != nil {
@@ -111,6 +111,7 @@ func newTCPConn(addr string, timeout time.Duration) (*net.TCPConn, error) {
 
 	done := make(chan tcpConn)
 	go func() {
+		defer log.PanicHandler()
 		addr, err := net.ResolveTCPAddr("tcp", addr)
 		if err != nil {
 			done <- tcpConn{nil, err}
@@ -129,6 +130,7 @@ func newTCPConn(addr string, timeout time.Duration) (*net.TCPConn, error) {
 	select {
 	case <-time.After(timeout):
 		go func() {
+			defer log.PanicHandler()
 			if tcpResult := <-done; tcpResult.conn != nil {
 				tcpResult.conn.Close()
 			}
