@@ -10,6 +10,7 @@ import (
 	"github.com/emersion/go-message/mail"
 	"github.com/emersion/go-message/textproto"
 
+	"git.sr.ht/~rjarry/aerc/lib/parse"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -41,6 +42,7 @@ func (imapw *IMAPWorker) handleFetchMessageHeaders(
 		imap.FetchInternalDate,
 		imap.FetchFlags,
 		imap.FetchUid,
+		imap.FetchRFC822Size,
 		section.FetchItem(),
 	}
 	imapw.handleFetchMessages(msg, toFetch, items,
@@ -64,12 +66,10 @@ func (imapw *IMAPWorker) handleFetchMessageHeaders(
 				Flags:         translateImapFlags(_msg.Flags),
 				InternalDate:  _msg.InternalDate,
 				RFC822Headers: header,
+				Refs:          parse.MsgIDList(header, "references"),
+				Size:          _msg.Size,
 				Uid:           _msg.Uid,
 			}
-			refs, err := header.MsgIDList("references")
-			if err != nil {
-				info.Refs = refs
-			}
 			imapw.worker.PostMessage(&types.MessageInfo{
 				Message: types.RespondTo(msg),
 				Info:    info,
