@@ -50,7 +50,13 @@ func parseSearch(args []string) (*searchCriteria, error) {
 		case 'X':
 			criteria.WithoutFlags = append(criteria.WithoutFlags, getParsedFlag(opt.Value))
 		case 'H':
-			// TODO
+			if strings.Contains(opt.Value, ": ") {
+				HeaderValue := strings.SplitN(opt.Value, ": ", 2)
+				criteria.Header.Add(HeaderValue[0], HeaderValue[1])
+			} else {
+				log.Errorf("Header is not given properly, must be given in format `Header: Value`")
+				continue
+			}
 		case 'f':
 			criteria.Header.Add("From", opt.Value)
 		case 't':
@@ -120,6 +126,7 @@ func (w *Worker) search(criteria *searchCriteria) ([]uint32, error) {
 		limit <- struct{}{}
 		wg.Add(1)
 		go func(key uint32) {
+			defer log.PanicHandler()
 			defer wg.Done()
 			success, err := w.searchKey(key, criteria, requiredParts)
 			if err != nil {
