@@ -14,9 +14,9 @@ import (
 	"sort"
 	"strings"
 	"sync"
+	"time"
 
 	"github.com/emersion/go-maildir"
-	"github.com/fsnotify/fsnotify"
 
 	aercLib "git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/iterator"
@@ -39,28 +39,48 @@ type Worker struct {
 	c                   *Container
 	selected            *maildir.Dir
 	selectedName        string
+	selectedInfo        *models.DirectoryInfo
 	worker              *types.Worker
-	watcher             *fsnotify.Watcher
+	watcher             types.FSWatcher
+	watcherDebounce     *time.Timer
+	fsEvents            chan struct{}
 	currentSortCriteria []*types.SortCriterion
 	maildirpp           bool // whether to use Maildir++ directory layout
+	capabilities        *models.Capabilities
 }
 
 // NewWorker creates a new maildir worker with the provided worker.
 func NewWorker(worker *types.Worker) (types.Backend, error) {
-	watch, err := fsnotify.NewWatcher()
+	watch, err := handlers.NewWatcher()
 	if err != nil {
 		return nil, fmt.Errorf("could not create file system watcher: %w", err)
 	}
-	return &Worker{worker: worker, watcher: watch}, nil
+	return &Worker{
+		capabilities: &models.Capabilities{
+			Sort:   true,
+			Thread: true,
+		},
+		worker:   worker,
+		watcher:  watch,
+		fsEvents: make(chan struct{}),
+	}, nil
 }
 
 // NewMaildirppWorker creates a new Maildir++ worker with the provided worker.
 func NewMaildirppWorker(worker *types.Worker) (types.Backend, error) {
-	watch, err := fsnotify.NewWatcher()
+	watch, err := handlers.NewWatcher()
 	if err != nil {
 		return nil, fmt.Errorf("could not create file system watcher: %w", err)
 	}
-	return &Worker{worker: worker, watcher: watch, maildirpp: true}, nil
+	return &Worker{
+		capabilities: &models.Capabilities{
+			Sort:   true,
+			Thread: true,
+		},
+		worker:    worker,
+		watcher:   watch,
+		maildirpp: true,
+	}, nil
 }
 
 // Run starts the worker's message handling loop.
@@ -69,12 +89,25 @@ func (w *Worker) Run() {
 		select {
 		case action := <-w.worker.Actions:
 			w.handleAction(action)
-		case ev := <-w.watcher.Events:
-			w.handleFSEvent(ev)
+		case <-w.watcher.Events():
+			if w.watcherDebounce != nil {
+				w.watcherDebounce.Stop()
+			}
+			// Debounce FS changes
+			w.watcherDebounce = time.AfterFunc(50*time.Millisecond, func() {
+				defer log.PanicHandler()
+				w.fsEvents <- struct{}{}
+			})
+		case <-w.fsEvents:
+			w.handleFSEvent()
 		}
 	}
 }
 
+func (w *Worker) Capabilities() *models.Capabilities {
+	return w.capabilities
+}
+
 func (w *Worker) handleAction(action types.WorkerMessage) {
 	msg := w.worker.ProcessAction(action)
 	switch msg := msg.(type) {
@@ -101,14 +134,7 @@ func (w *Worker) handleAction(action types.WorkerMessage) {
 	}
 }
 
-func (w *Worker) handleFSEvent(ev fsnotify.Event) {
-	// we only care about files being created, removed or renamed
-	switch ev.Op {
-	case fsnotify.Create, fsnotify.Remove, fsnotify.Rename:
-		break
-	default:
-		return
-	}
+func (w *Worker) handleFSEvent() {
 	// if there's not a selected directory to rescan, ignore
 	if w.selected == nil {
 		return
@@ -119,9 +145,10 @@ func (w *Worker) handleFSEvent(ev fsnotify.Event) {
 		return
 	}
 
-	dirInfo := w.getDirectoryInfo(w.selectedName)
+	w.selectedInfo = w.getDirectoryInfo(w.selectedName)
 	w.worker.PostMessage(&types.DirectoryInfo{
-		Info: dirInfo,
+		Info:    w.selectedInfo,
+		Refetch: true,
 	}, nil)
 }
 
@@ -168,22 +195,13 @@ func dirFiles(name string) ([]string, error) {
 
 func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 	dirInfo := &models.DirectoryInfo{
-		Name:     name,
-		Flags:    []string{},
-		ReadOnly: false,
+		Name: name,
 		// total messages
 		Exists: 0,
 		// new messages since mailbox was last opened
 		Recent: 0,
 		// total unread
 		Unseen: 0,
-
-		AccurateCounts: false,
-
-		Caps: &models.Capabilities{
-			Sort:   true,
-			Thread: true,
-		},
 	}
 
 	dir := w.c.Store.Dir(name)
@@ -250,7 +268,6 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 			dirInfo.Recent++
 		}
 	}
-	dirInfo.AccurateCounts = true
 	return dirInfo
 }
 
@@ -321,6 +338,10 @@ func (w *Worker) handleConfigure(msg *types.Configure) error {
 		return err
 	}
 	w.c = c
+	err = w.watcher.Configure(dir)
+	if err != nil {
+		return err
+	}
 	log.Debugf("configured base maildir: %s", dir)
 	return nil
 }
@@ -345,8 +366,7 @@ func (w *Worker) handleListDirectories(msg *types.ListDirectories) error {
 		w.worker.PostMessage(&types.Directory{
 			Message: types.RespondTo(msg),
 			Dir: &models.Directory{
-				Name:       name,
-				Attributes: []string{},
+				Name: name,
 			},
 		}, nil)
 
@@ -398,6 +418,7 @@ func (w *Worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 	info := &types.DirectoryInfo{
 		Info: w.getDirectoryInfo(msg.Directory),
 	}
+	w.selectedInfo = info.Info
 	w.worker.PostMessage(info, nil)
 	return nil
 }
@@ -458,6 +479,7 @@ func (w *Worker) sort(uids []uint32, criteria []*types.SortCriterion) ([]uint32,
 		limit <- struct{}{}
 		wg.Add(1)
 		go func(uid uint32) {
+			defer log.PanicHandler()
 			defer wg.Done()
 			info, err := w.msgHeadersFromUid(uid)
 			if err != nil {
@@ -528,6 +550,7 @@ func (w *Worker) threads(uids []uint32, criteria []*types.SortCriterion) ([]*typ
 		limit <- struct{}{}
 		wg.Add(1)
 		go func(uid uint32) {
+			defer log.PanicHandler()
 			defer wg.Done()
 			info, err := w.msgHeadersFromUid(uid)
 			if err != nil {
@@ -712,7 +735,7 @@ func (w *Worker) handleFlagMessages(msg *types.FlagMessages) error {
 			w.err(msg, err)
 			continue
 		}
-		flag := lib.FlagToMaildir[msg.Flag]
+		flag := lib.FlagToMaildir[msg.Flags]
 		if err := m.SetOneFlag(flag, msg.Enable); err != nil {
 			log.Errorf("could change flag %v to %v on message: %v", flag, msg.Enable, err)
 			w.err(msg, err)
@@ -817,7 +840,7 @@ func (w *Worker) msgInfoFromUid(uid uint32) (*models.MessageInfo, error) {
 		return nil, err
 	}
 	if w.c.IsRecent(uid) {
-		info.Flags = append(info.Flags, models.RecentFlag)
+		info.Flags |= models.RecentFlag
 	}
 	return info, nil
 }
@@ -835,6 +858,7 @@ func (w *Worker) msgHeadersFromUid(uid uint32) (*models.MessageInfo, error) {
 }
 
 func (w *Worker) handleCheckMail(msg *types.CheckMail) {
+	defer log.PanicHandler()
 	if msg.Command == "" {
 		w.err(msg, fmt.Errorf("checkmail: no command specified"))
 		return
@@ -844,6 +868,7 @@ func (w *Worker) handleCheckMail(msg *types.CheckMail) {
 	cmd := exec.CommandContext(ctx, "sh", "-c", msg.Command)
 	ch := make(chan error)
 	go func() {
+		defer log.PanicHandler()
 		err := cmd.Run()
 		ch <- err
 	}()
@@ -865,8 +890,7 @@ func (w *Worker) handleCheckMail(msg *types.CheckMail) {
 				}
 				dirInfo := w.getDirectoryInfo(name)
 				w.worker.PostMessage(&types.DirectoryInfo{
-					Info:     dirInfo,
-					SkipSort: true,
+					Info: dirInfo,
 				}, nil)
 			}
 			w.done(msg)
