@@ -5,8 +5,4 @@ package notmuch
 
 type eventType interface{}
 
-type event struct{}
-
-type updateDirCounts struct {
-	event
-}
+type updateDirCounts struct{}
