@@ -12,6 +12,7 @@ import (
 
 	"github.com/emersion/go-maildir"
 
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
 	notmuch "git.sr.ht/~rjarry/aerc/worker/notmuch/lib"
@@ -34,7 +35,19 @@ func (m *Message) NewReader() (io.ReadCloser, error) {
 
 // MessageInfo populates a models.MessageInfo struct for the message.
 func (m *Message) MessageInfo() (*models.MessageInfo, error) {
-	return lib.MessageInfo(m)
+	info, err := lib.MessageInfo(m)
+	if err != nil {
+		return nil, err
+	}
+	// if size retrieval fails, just return info and log error
+	if name, err := m.Filename(); err != nil {
+		log.Errorf("failed to obtain filename: %v", err)
+	} else {
+		if info.Size, err = lib.FileSize(name); err != nil {
+			log.Errorf("failed to obtain file size: %v", err)
+		}
+	}
+	return info, nil
 }
 
 // NewBodyPartReader creates a new io.Reader for the requested body part(s) of
@@ -58,7 +71,7 @@ func (m *Message) NewBodyPartReader(requestedParts []int) (io.Reader, error) {
 
 // SetFlag adds or removes a flag from the message.
 // Notmuch doesn't support all the flags, and for those this errors.
-func (m *Message) SetFlag(flag models.Flag, enable bool) error {
+func (m *Message) SetFlag(flag models.Flags, enable bool) error {
 	// Translate the flag into a notmuch tag, ignoring no-op flags.
 	var tag string
 	switch flag {
@@ -129,28 +142,22 @@ func (m *Message) Labels() ([]string, error) {
 	return m.Tags()
 }
 
-func (m *Message) ModelFlags() ([]models.Flag, error) {
-	var flags []models.Flag
-	seen := true
+func (m *Message) ModelFlags() (models.Flags, error) {
+	var flags models.Flags = models.SeenFlag
 	tags, err := m.Tags()
 	if err != nil {
-		return nil, err
+		return 0, err
 	}
 	for _, tag := range tags {
 		switch tag {
 		case "replied":
-			flags = append(flags, models.AnsweredFlag)
+			flags |= models.AnsweredFlag
 		case "flagged":
-			flags = append(flags, models.FlaggedFlag)
+			flags |= models.FlaggedFlag
 		case "unread":
-			seen = false
-		default:
-			continue
+			flags &^= models.SeenFlag
 		}
 	}
-	if seen {
-		flags = append(flags, models.SeenFlag)
-	}
 	return flags, nil
 }
 
